﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using Steema.TeeChart.Drawing;
using Steema.TeeChart.Styles;

namespace HowToCustomDrawShape
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 그리기 전 여부
        /// </summary>
        private bool beforeDraw;

        /// <summary>
        /// 축 그리기 전 여부
        /// </summary>
        private bool beforeDrawAxis;

        /// <summary>
        /// 시리즈 그리기 전 여부
        /// </summary>
        private bool beforeDrawSeries; 

        /// <summary>
        /// 그리기 후 여부
        /// </summary>
        private bool afterDraw;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.tChart.Aspect.View3D = false;

            this.tChart.BeforeDraw       += tChart_BeforeDraw;
            this.tChart.BeforeDrawAxes   += tChart_BeforeDrawAxes;
            this.tChart.BeforeDrawSeries += tChart_BeforeDrawSeries;
            this.tChart.AfterDraw        += tChart_AfterDraw;

            this.beforeDrawRadioButton.CheckedChanged       += beforeDrawRadioButton_CheckedChanged;
            this.beforeDrawAxisRadioButton.CheckedChanged   += beforeDrawAxisRadioButton_CheckedChanged;
            this.beforeDrawSeriesRadioButton.CheckedChanged += beforeDrawSeriesRadioButton_CheckedChanged;
            this.afterDrawRadioButton.CheckedChanged        += afterDrawRadioButton_CheckedChanged;

            Load += Form_Load;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            SetFlag(ref this.beforeDraw);
    
            Bar bar = new Bar(this.tChart.Chart);
    
            bar.FillSampleValues(20);
    
            beforeDrawRadioButton.Checked = true;
        }

        #endregion
        #region 티차트 그리기 전 처리하기 - tChart_BeforeDraw(sender, graphics3D)

        /// <summary>
        /// 티차트 그리기 전 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="graphics3D">Graphics3D 객체</param>
        private void tChart_BeforeDraw(object sender, Graphics3D graphics3D)
        {
            if(this.beforeDraw)
            {
                DrawShape(graphics3D);
            }
        }

        #endregion
        #region 티차트 축 그리기 전 처리하기 - tChart_BeforeDrawAxes(sender, graphics3D)

        /// <summary>
        /// 티차트 축 그리기 전 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="graphics3D">Graphics3D 객체</param>
        private void tChart_BeforeDrawAxes(object sender, Graphics3D graphics3D)
        {
            if(this.beforeDrawAxis)
            {
                DrawShape(graphics3D);
            }
        }

        #endregion
        #region 티차트 시리즈 그리기 전 처리하기 - tChart_BeforeDrawSeries(sender, graphics3D)

        /// <summary>
        /// 티차트 시리즈 그리기 전 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="graphics3D">Graphics3D 객체</param>
        private void tChart_BeforeDrawSeries(object sender, Graphics3D graphics3D)
        {
            if(this.beforeDrawSeries)
            {
                DrawShape(graphics3D);
            }
        }

        #endregion
        #region 티차트 그리기 후 처리하기 - tChart_AfterDraw(sender, graphics3D)

        /// <summary>
        /// 티차트 그리기 후 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="graphics3D">Graphics3D 객체</param>
        private void tChart_AfterDraw(object sender, Graphics3D graphics3D)
        {
            if(this.afterDraw)
            {
                DrawShape(graphics3D);
            }
        }

        #endregion

        #region Before Draw 라디오 버튼 체크 변경시 처리하기 - beforeDrawRadioButton_CheckedChanged(sender, e)

        /// <summary>
        /// Before Draw 라디오 버튼 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void beforeDrawRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if(this.beforeDrawRadioButton.Checked)
            {
                SetFlag(ref this.beforeDraw);
    
                this.tChart.Refresh();
            }
        } 

        #endregion
        #region Before Draw Axis 라디오 버튼 체크 변경시 처리하기 - beforeDrawAxisRadioButton_CheckedChanged(sender, e)

        /// <summary>
        /// Before Draw Axis 라디오 버튼 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void beforeDrawAxisRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if(this.beforeDrawAxisRadioButton.Checked)
            {
                SetFlag(ref this.beforeDrawAxis);
    
                this.tChart.Refresh();
            }
        }

        #endregion
        #region Before Draw Series 라디오 버튼 체크 변경시 처리하기 - beforeDrawSeriesRadioButton_CheckedChanged(sender, e)

        /// <summary>
        /// Before Draw Series 라디오 버튼 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void beforeDrawSeriesRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if(this.beforeDrawSeriesRadioButton.Checked)
            {
                SetFlag(ref this.beforeDrawSeries);
    
                this.tChart.Refresh();
            }
        }

        #endregion
        #region After Draw 라디오 버튼 체크 변경시 처리하기 - afterDrawRadioButton_CheckedChanged(sender, e)

        /// <summary>
        /// After Draw 라디오 버튼 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void afterDrawRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if(this.afterDrawRadioButton.Checked)
            {
                SetFlag(ref this.afterDraw);
    
                this.tChart.Refresh();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 플래그 설정하기 - SetFlag(flag)

        /// <summary>
        /// 플래그 설정하기
        /// </summary>
        /// <param name="flag">플래그 여부</param>
        private void SetFlag(ref bool flag)
        {
            this.beforeDraw       = false;
            this.afterDraw        = false;
            this.beforeDrawAxis   = false;
            this.beforeDrawSeries = false;

            flag = true;
        }

        #endregion
        #region 도형 그리기 - DrawShape(graphics3D)

        /// <summary>
        /// 도형 그리기
        /// </summary>
        /// <param name="graphics3D">Graphics3D 객체</param>
        private void DrawShape(Graphics3D graphics3D)
        {
            graphics3D.Brush.Visible = true;
            graphics3D.Brush.Color   = Color.Yellow;
            graphics3D.Pen.Visible   = true;
            graphics3D.Pen.Style     = DashStyle.Dash;

            graphics3D.Ellipse(1, 1, graphics3D.Chart.Width - 1, graphics3D.Chart.Height - 1);
        }

        #endregion
    }
}